#include <jni.h>
#include <android/log.h>
#include "scripting/lua-bindings/manual/CCLuaEngine.h"

static const char *TAG = "AdjoeJNI";

static lua_State *get_lua_state() {
    auto engine = cocos2d::LuaEngine::getInstance();
    cocos2d::ScriptEngineManager::getInstance()->setScriptEngine(engine);
    return engine->getLuaStack()->getLuaState();
}

extern "C"
JNIEXPORT void JNICALL
Java_io_adjoe_cocos2dx_AdjoeSDKModule_doLuaCallback(JNIEnv *env, jclass, jint function_reference,
                                                    jstring arg) {
    lua_State *L = get_lua_state();
    lua_rawgeti(L, LUA_REGISTRYINDEX, (int) function_reference);
    if (arg != nullptr) {
        const char *resultMessage = env->GetStringUTFChars(arg, nullptr);
        lua_pushstring(L, resultMessage);
        env->ReleaseStringUTFChars(arg, resultMessage);
    }
    if (lua_pcall(L, arg != nullptr ? 1 : 0, 0, 0) != 0) {
        __android_log_print(ANDROID_LOG_ERROR, TAG, "Failed to call callback.\n %s\n",
                            lua_tostring(L, -1));
    }
}


extern "C"
JNIEXPORT void JNICALL
Java_io_adjoe_cocos2dx_AdjoeSDKModule_freeCallbackReferences(JNIEnv *env, jclass clazz,

                                                             jintArray function_reference) {
    lua_State *L = get_lua_state();
    jsize len = env->GetArrayLength(function_reference);
    jint *items = env->GetIntArrayElements(function_reference, nullptr);
    for (jsize i = 0; i < len; i++) {
        luaL_unref(L, LUA_REGISTRYINDEX, (int) items[i]);
    }
}

extern "C"
JNIEXPORT void JNICALL
Java_io_adjoe_cocos2dx_AdjoeSDKModule_doLuaRewardCallback(JNIEnv *env, jclass clazz,
                                                          jint function_reference,
                                                          jobject response) {
    lua_State *L = get_lua_state();
    lua_rawgeti(L, LUA_REGISTRYINDEX, (int) function_reference);
    if (response != nullptr) {
        jclass responseClass = env->FindClass("io/adjoe/sdk/AdjoeRewardResponse");
        jmethodID getRewardMethod = env->GetMethodID(responseClass, "getReward", "()I");
        jmethodID getAvailableCoinsMethod = env->GetMethodID(responseClass,
                                                             "getAvailablePayoutCoins", "()I");
        jmethodID getAlreadySpentCoinsMethod = env->GetMethodID(responseClass,
                                                                "getAlreadySpentCoins",
                                                                "()I");

        jint rewards = env->CallIntMethod(response, getRewardMethod);
        jint availableCoins = env->CallIntMethod(response, getAvailableCoinsMethod);
        jint alreadySpentCoins = env->CallIntMethod(response, getAlreadySpentCoinsMethod);

        lua_createtable(L, 0, 3);
        lua_pushinteger(L, (int) rewards);
        lua_setfield(L, -2, "rewards");

        lua_pushinteger(L, (int) availableCoins);
        lua_setfield(L, -2, "availableCoins");

        lua_pushinteger(L, (int) alreadySpentCoins);
        lua_setfield(L, -2, "alreadySpentCoins");
    }
    if (lua_pcall(L, 1, 0, 0) != 0) {
        __android_log_print(ANDROID_LOG_ERROR, TAG, "Failed to call reward callback.\n %s\n",
                            lua_tostring(L, -1));
    }
}

extern "C"
JNIEXPORT void JNICALL
Java_io_adjoe_cocos2dx_AdjoeSDKModule_doLuaPayoutSuccessCallback(JNIEnv *env, jclass clazz,
                                                                 jint function_reference,
                                                                 jint coins) {
    lua_State *L = get_lua_state();
    lua_rawgeti(L, LUA_REGISTRYINDEX, (int) function_reference);
    lua_createtable(L, 0, 1);
    lua_pushinteger(L, (int) coins);
    lua_setfield(L, -2, "coins");
    if (lua_pcall(L, 1, 0, 0) != 0) {
        __android_log_print(ANDROID_LOG_ERROR, TAG, "Failed to call payout callback.\n %s\n",
                            lua_tostring(L, -1));
    }
}


extern "C"
JNIEXPORT void JNICALL
Java_io_adjoe_cocos2dx_AdjoeSDKModule_doLuaPayoutErrorCallback(JNIEnv *env, jclass clazz,
                                                               jint function_reference, jint reason,
                                                               jstring message) {
    lua_State *L = get_lua_state();
    lua_rawgeti(L, LUA_REGISTRYINDEX, (int) function_reference);
    lua_createtable(L, 0, 2);
    lua_pushinteger(L, (int) reason);
    lua_setfield(L, -2, "reason");

    if (lua_pcall(L, 1, 0, 0) != 0) {
        __android_log_print(ANDROID_LOG_ERROR, TAG, "Failed to call payout callback.\n %s\n",
                            lua_tostring(L, -1));
    }

    if (message != nullptr) {
        const char *resultMessage = env->GetStringUTFChars(message, nullptr);
        lua_pushstring(L, resultMessage);
        env->ReleaseStringUTFChars(message, resultMessage);
    } else {
        lua_pushnil(L);
    }
    lua_setfield(L, -2, "message");
    if (lua_pcall(L, 1, 0, 0) != 0) {
        __android_log_print(ANDROID_LOG_ERROR, TAG, "Failed to call payout callback.\n %s\n",
                            lua_tostring(L, -1));
    }
}
