local Adjoe = class("Adjoe")
local Options = class("Options")
local AdjoeParams = class("AdjoeParams")

-- Constants
-- The user sees the Terms of Service of your app.
Adjoe.EVENT_TOS_SHOWN = 1

-- The user has just accepted the Terms of Service of your app.
-- Trigger this event when the user has pressed "yes" (or other positive action) on the dialog asking for his agreement to the adjoe Terms of Service.
Adjoe.EVENT_TOS_ACCEPTED = 2

-- The user has just declined the Terms of Service of your app.
-- Trigger this event when the user has pressed "no" (or other negative action) on the dialog asking for his agreement to the adjoe Terms of Service.
Adjoe.EVENT_TOS_DECLINED = 3

-- The user has just given your app the access to the usage data.
-- Trigger this event when the user has returned from his phone settings to your app for the first time with the permission for usage tracking via your app given.
Adjoe.EVENT_USAGE_PERMISSION_ACCEPTED = 4

-- The user has denied to give your app the access to the usage data.
-- Trigger this event when the user has pressed "no" (or other negative action) on the dialog asking for permission for usage tracking.
Adjoe.EVENT_USAGE_PERMISSION_DENIED = 5

-- The user has started the video for any of the partner apps.
-- Trigger this event when the user has started the video of an adjoe advertisement.
Adjoe.EVENT_VIDEO_PLAY = 7

-- The user has paused the video for any of the partner apps.
-- Trigger this event when the user has paused the video of an adjoe advertisement.
Adjoe.EVENT_VIDEO_PAUSE = 8

-- The video for one of the partner apps has ended.
-- Trigger this event when the video of an adjoe advertisement has reached its end.
Adjoe.EVENT_VIDEO_ENDED = 9

-- You have just displayed the campaigns screen to the user.
-- Trigger this event when the adjoe offerwall is loaded with either advertised partner apps ir already installed partner apps for signed-up users.
Adjoe.EVENT_CAMPAIGNS_SHOWN = 10

-- The user has opened your app.
-- Trigger this event when your app is launched, i.e. a new SDK app session for the user is initiated.
Adjoe.EVENT_APP_OPEN = 12

-- The user sees the adjoe content (e.g. the Start method of the Scene displaying the adjoe content is called).
-- Trigger this event on the following actions:
-- (1) the user has opened your app and started a new adjoe-related session.
-- (2) the user has brought your app from the background to the foreground to continue his adjoe-related session (exception: the user has left your app to allow usage tracking in his phone settings).</item>
-- This event should be triggered in both cases regardless of whether the user has already signed up or not.
Adjoe.EVENT_FIRST_IMPRESSION = 13

-- The user can see the teaser, e.g. the button via which he can access the adjoe SDK from the SDK app.
-- Trigger this event when the teaser has been successfully rendered and would successfully redirect the user to the adjoe SDK.
-- it should be triggered regardless of whether the user has actually clicked the teaser or not. This event is mostly appropriate for uses in which the functionality of the SDK app and SDK are kept separate to a relevant degree.
Adjoe.EVENT_TEASER_SHOWN = 14

-- An unspecified payout error.
Adjoe.PAYOUT_ERR_UNKNOWN = 0

-- The user has not accepted the Terms of Service (TOS).
Adjoe.PAYOUT_ERR_TOS_NOT_ACCEPTED = 1

-- The user has not collected enough rewards for a payout.
Adjoe.PAYOUT_ERR_NOT_ENOUGH_COINS = 400

-- Basic APIs

function Adjoe.Init(sdkHash, options, success, failure)
    adjoec_init(sdkHash, options, success, failure)
end

function Adjoe.IsInitialized()
    return adjoec_is_initialized()
end

function Adjoe.ShowOfferwall(params)
    adjoec_show_ow(params)
end

function Adjoe.RequestRewards(params, success, failure)
    adjoec_request_rewards(params, success, failure)
end

function Adjoe.SendUserEvent(event, params)
    adjoec_send_user_event(event, params)
end

function Adjoe.DoPayout(params, success, failure)
    adjoec_do_payout(params, success, failure)
end

function Adjoe.SetProfile(source, gender, birthday, params)
    adjoec_set_profile(source, gender, birthday, params)
end

function Adjoe.CanShowOfferwall()
    return adjoec_can_show_ow()
end

function Adjoe.GetVersion()
    return adjoec_get_version()
end

function Adjoe.GetVersionName()
    return adjoec_get_version_name()
end

function Adjoe.HasAcceptedTOS()
    return adjoec_has_accepted_tos()
end

function Adjoe.HasAcceptedUsagePermission()
    return adjoec_has_accepted_usage_permission()
end

-- Adjoe Options and Parameter Classes

function Adjoe.NewOptions()
    local self = setmetatable({}, Options)
    self.UserID = nil
    self.AdjoeParams = nil
    return self
end

function Options:SetUserID(UserID)
    self.UserID = UserID
    return self
end

function Options:SetAdjoeParams(AdjoeParams)
    self.AdjoeParams = AdjoeParams
    return self
end

function Adjoe.NewAdjoeParams()
    local self = setmetatable({}, AdjoeParams)
    self.UANetwork = nil
    self.UAChannel = nil
    self.UASubPublisherCleartext = nil
    self.UASubPublisherEncrypted = nil
    self.Placement = nil
    return self
end

function AdjoeParams:SetUANetwork(UANetwork)
    self.UANetwork = UANetwork
    return self
end

function AdjoeParams:SetUAChannel(UAChannel)
    self.UAChannel = UAChannel
    return self
end

function AdjoeParams:SetUASubPublisherCleartext(UASubPublisherCleartext)
    self.UASubPublisherCleartext = UASubPublisherCleartext
    return self
end

function AdjoeParams:SetUASubPublisherEncrypted(UASubPublisherEncrypted)
    self.UASubPublisherEncrypted = UASubPublisherEncrypted
    return self
end

function AdjoeParams:SetPlacement(Placement)
    self.Placement = Placement
    return self
end

return Adjoe
